@@ -1,5 +1,6 @@
 from music21 import stream, note, tempo, meter, instrument
 from music21 import duration as m21duration
+import re
 
 class Drummer:
     def __init__(self, bpm=120, volume=100, accent=20, signature='4/4'):
@@ -12,14 +13,18 @@ class Drummer:
             'kick': { 'instrument': 'kick1', 'part': stream.Part(), 'counter': 0 },
             'snare': { 'instrument': 'snare1', 'part': stream.Part(), 'counter': 0 },
             'hihat': { 'instrument': 'hihat1', 'part': stream.Part(), 'counter': 0 },
+            'toms': { 'instrument': 'tom3', 'part': stream.Part(), 'counter': 0 },
+            'cymbals': { 'instrument': 'crash1', 'part': stream.Part(), 'counter': 0 },
+            'percussion': { 'instrument': 'woodblock1', 'part': stream.Part(), 'counter': 0 },
         }
         self.score = stream.Score()
 
     def sync_parts(self):
         for part in self.kit.values():
             patch = self.instrument_map(part['instrument'])
-            self.score.insert(0, part['part'])
-            part['part'].insert(0, patch['obj'])
+            if len(part['part'].getElementsByClass('Note')) > 0:
+                self.score.insert(0, part['part'])
+                part['part'].insert(0, patch['obj'])
 
     def set_ts(self, ts=None):
         if not ts:
@@ -43,31 +48,25 @@ class Drummer:
             self.kit[name] = { 'instrument': patch, 'part': stream.Part(), 'counter': 0 }
 
     def rest(self, name, duration=1.0):
-        if isinstance(name, list):
-            for item in name:
-                if item == 'closed':
-                    item = 'hihat'
-                elif item == 'open':
-                    item = 'hihat'
-                elif item == 'pedal':
-                    item = 'hihat'
-                n = note.Rest()
-                n.duration = m21duration.Duration(duration)
-                self.kit[item]['part'].append(n)
-                if duration:
-                    self.kit[item]['counter'] += duration
-        else:
-            if name == 'closed':
-                name = 'hihat'
-            elif name == 'open':
-                name = 'hihat'
-            elif name == 'pedal':
-                name = 'hihat'
+        if not isinstance(name, list):
+            name = [name]
+        for item in name:
+            if item == 'closed':
+                item = 'hihat'
+            elif item == 'open':
+                item = 'hihat'
+            elif item == 'pedal':
+                item = 'hihat'
+            elif re.search(r"^tom\d$", item):
+                item = 'toms'
+            elif re.search(r"^ride.+$", item) or re.search(r"^crash\d$", item) or item == 'china' or item == 'splash':
+                item = 'cymbals'
+            elif re.search(r"^bongo\d$", item) or re.search(r"^conga\d$", item) or re.search(r"^timbale\d$", item) or re.search(r"^woodblock\d$", item) or re.search(r"^triangle\d$", item) or item == 'clap' or item == 'cowbell' or item == 'tambourine' or item == 'shaker' or item == 'cabasa' or item == 'claves':
+                item = 'percussion'
             n = note.Rest()
             n.duration = m21duration.Duration(duration)
-            self.kit[name]['part'].append(n)
-            if duration:
-                self.kit[name]['counter'] += duration
+            self.kit[item]['part'].append(n)
+            self.kit[item]['counter'] += duration
 
     def note(self, name, duration=1.0, volume=None, flam=0):
         if name == 'hihat' or name == 'closed':
@@ -79,6 +78,15 @@ class Drummer:
         elif name == 'pedal':
             patch = self.instrument_map('hihat3')
             name = 'hihat'
+        elif re.search(r"^tom\d$", name):
+            patch = self.instrument_map(name)
+            name = 'toms'
+        elif re.search(r"^ride.+$", name) or re.search(r"^crash\d$", name) or name == 'china' or name == 'splash':
+            patch = self.instrument_map(name)
+            name = 'cymbals'
+        elif re.search(r"^bongo\d$", name) or re.search(r"^conga\d$", name) or re.search(r"^timbale\d$", name) or re.search(r"^woodblock\d$", name) or re.search(r"^triangle\d$", name) or name == 'clap' or name == 'cowbell' or name == 'tambourine' or name == 'shaker' or name == 'cabasa' or name == 'claves':
+            patch = self.instrument_map(name)
+            name = 'percussion'
         else:
             patch = self.instrument_map(self.kit[name]['instrument'])
         if volume is None:
@@ -92,8 +100,7 @@ class Drummer:
         n.volume.velocity = volume
         n.duration = m21duration.Duration(duration - flam)
         self.kit[name]['part'].append(n)
-        if duration:
-            self.kit[name]['counter'] += duration
+        self.kit[name]['counter'] += duration
 
     def accent_note(self, name, duration=1.0, volume=None, accent=None):
         if volume is None:
